@@ -18,11 +18,8 @@ class NgramCorpus:
         MI, Xsq, Gsq, Dice, DeltaP21, DeltaP12, FisherExact
     ]
 
-    def __init__(self, corpus_reader=None, corpus=None, db_dir='dcctk.db/'):
+    def __init__(self, corpus_reader, db_dir='dcctk.db/'):
         self.corpus_reader = corpus_reader
-        self.corpus_mem = None
-        if isinstance(corpus, list):
-            self.corpus_mem = corpus
         self.n_subcorp = corpus_reader.n_subcorp
         self.database = {}
         self.pat_ch_chr = re.compile("[〇一-\u9fff㐀-\u4dbf豈-\ufaff]")
@@ -40,9 +37,7 @@ class NgramCorpus:
     def corpus(self):
         """Return a corpus as a generator
         """
-        if self.corpus_mem is None:
-            return self.corpus_reader.get_corpus_as_gen()
-        return self.corpus_mem
+        return self.corpus_reader.get_corpus_as_gen()
 
 
     def bigram_associations(self, subcorp_idx=None, chinese_only=True, 
@@ -267,6 +262,68 @@ class TextBasedCorpus:
                 self.path_index[text['id']] = (i, j)
 
 
+    def bigram_associations(self, subcorp_idx=None, chinese_only=True, 
+        sort_by="Gsq", reverse=True, fq_thresh=0):
+        distr = self.freq_distr_ngrams(2, subcorp_idx, chinese_only)
+        N = sum(distr.values())
+        R1 = Counter()
+        C1 = Counter()
+        for w1w2, fq in distr.items():
+            w1, w2 = w1w2[0], w1w2[1]
+            R1.update({w1: fq})
+            C1.update({w2: fq})
+        
+        output = []
+        for w1w2, o11 in distr.items():
+            if o11 < fq_thresh: continue
+            w1, w2 = w1w2[0], w1w2[1]
+            r1 = R1.get(w1, 0)
+            r2 = N - r1
+            c1 = C1.get(w2, 0)
+            o12 = r1 - o11
+            o21 = c1 - o11
+            o22 = r2 - o21
+            o11_raw = o11
+            o11, o12, o21, o22, e11, e12, e21, e22 = \
+                additive_smooth(o11_raw, o12, o21, o22, alpha=0)
+            stats = { 
+                func.__name__: func(o11, o12, o21, o22, e11, e12, e21, e22)\
+                    for func in self.association_measures
+            }
+            stats['RawCount'] = o11_raw
+            output.append((w1w2, stats))
+
+        return sorted(output, reverse=reverse, key=lambda x: x[1][sort_by])
+
+
+    def freq_distr_ngrams(self, n, subcorp_idx=None, chinese_only=True):
+        if (not hasattr(self, 'ngrams')) or (n not in self.ngrams):
+            self._count_ngrams(n)
+        if isinstance(subcorp_idx, int):
+            distr =  self.ngrams[n].get(subcorp_idx, Counter())
+        else:
+            for i, key in enumerate(self.ngrams[n]):
+                if i == 0: distr = self.ngrams[n][key].copy()
+                else: distr.update(self.ngrams[n][key])
+        if chinese_only: 
+            for k in list(distr.keys()):
+                if sum(1 for ch in k if self.pat_ch_chr.search(ch)) < n:
+                    del distr[k]
+        return distr
+
+
+    def _count_ngrams(self, n):
+        print(f'Counting {n}-grams...')
+        if not hasattr(self, 'ngrams'): self.ngrams = {}
+        self.ngrams[n] = {}
+        for i in trange(len(self.corpus)):
+            self.ngrams[n][i] = Counter()
+            for text in self.corpus[i]['text']:
+                for sent in text['c']:
+                    for ngram in ngrams(sent, n=n):
+                        ng = ''.join(ngram)
+                        self.ngrams[n][i].update({ng: 1})
+
 
 class IndexedCorpus(TextBasedCorpus):
     """Corpus object for fast concordance search
